@@ -18,6 +18,7 @@
 #include "chxj_hdml.h"
 #include "chxj_dump.h"
 #include "chxj_img_conv.h"
+#include "chxj_qr_code.h"
 
 static char* chtml10_node_exchange    (Chtml10* chtml, Node* node, int indent);
 static char* chtml10_start_html_tag   (Chtml10* chtml, Node* child);
@@ -81,10 +82,23 @@ chxj_exchange_chtml10(
   Chtml10     chtml10;
   Doc       doc;
 
+  /*--------------------------------------------------------------------------*/
+  /* If qrcode xml                                                            */
+  /*--------------------------------------------------------------------------*/
+  *dstlen = srclen;
+  dst = chxj_qr_code_blob_handler(r, src, (size_t*)dstlen);
+  if (dst != NULL)
+  {
+    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"i found qrcode xml");
+    return dst;
+  }
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"not found qrcode xml");
+
   /*--------------------------------------------------------------------------*/
   /* The CHTML structure is initialized.                                      */
   /*--------------------------------------------------------------------------*/
   chxj_init_chtml10(&chtml10, &doc, r, spec);
+  ap_set_content_type(r, "text/html; charset=Windows-31J");
 
   /*--------------------------------------------------------------------------*/
   /* The character string of the input is analyzed.                           */
@@ -347,7 +361,16 @@ chtml10_node_exchange(Chtml10* chtml10, Node* node, int indent)
       if (chxj_chxjif_is_mine(chtml10->spec, doc, child))
       {
         ap_log_rerror(APLOG_MARK, APLOG_DEBUG,0,r, "chxj:if tag is mine");
-        chtml10_chxjif_tag(chtml10, child);
+        char* parse_attr = NULL;
+        parse_attr = qs_get_parse_attr(doc, child, r);
+        if (parse_attr != NULL && strcasecmp(parse_attr, "true") == 0)
+        {
+          chtml10_node_exchange (chtml10, child, indent+1);
+        }
+        else
+        {
+          chtml10_chxjif_tag(chtml10, child);
+        }
       }
     }
     /*------------------------------------------------------------------------*/
