@@ -20,6 +20,7 @@
 #include <unistd.h>
 #include "chxj_img_conv.h"
 #include "chxj_dump.h"
+#include "chxj_qr_code.h"
 
 #define CHECK_BOX_PREFIX     "_chxj_c_"
 #define RADIO_BUTTON_PREFIX  "_chxj_r_"
@@ -97,11 +98,23 @@ chxj_exchange_hdml(request_rec* r,
   Doc       doc;
   Hdml      hdml;
 
+  /*--------------------------------------------------------------------------*/
+  /* If qrcode xml                                                            */
+  /*--------------------------------------------------------------------------*/
+  *dstlen = srclen;
+  dst = chxj_qr_code_blob_handler(r, src, (size_t*)dstlen);
+  if (dst != NULL)
+  {
+    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"i found qrcode xml");
+    return dst;
+  }
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"not found qrcode xml");
+
   /*--------------------------------------------------------------------------*/
   /* initialize hdml structure                                                */
   /*--------------------------------------------------------------------------*/
   chxj_init_hdml(&hdml,&doc,r, spec);
-
+  ap_set_content_type(r, "text/x-hdml; charset=Shift_JIS");
   /*--------------------------------------------------------------------------*/
   /* DEBUG                                                                    */
   /*--------------------------------------------------------------------------*/
@@ -457,7 +470,16 @@ hdml_1_0_node_exchange(Hdml* hdml, Node* node,  int indent)
       if (chxj_chxjif_is_mine(hdml->spec, doc, child))
       {
         ap_log_rerror(APLOG_MARK, APLOG_DEBUG,0,r, "chxj:if tag is mine");
-        hdml_1_0_chxjif_tag(hdml, child);
+        char* parse_attr = NULL;
+        parse_attr = qs_get_parse_attr(doc, child, r);
+        if (parse_attr != NULL && strcasecmp(parse_attr, "true") == 0)
+        {
+          hdml->out = hdml_1_0_node_exchange(hdml, child,indent+1);
+        }
+        else
+        {
+          hdml_1_0_chxjif_tag(hdml, child);
+        }
       }
     }
     /*------------------------------------------------------------------------*/
