@@ -19,6 +19,8 @@
 #include "chxj_img_conv_format.h"
 #include "chxj_specified_device.h"
 #include "chxj_str_util.h"
+#include "chxj_qr_code.h"
+#include "qs_parse_string.h"
 
 #include "http_core.h"
 #include <wand/magick_wand.h>
@@ -39,16 +41,55 @@ typedef enum img_conv_mode_t {
   IMG_CONV_MODE_EZGET,
 } img_conv_mode_t;
 
+/*----------------------------------------------------------------------------*/
+/* User-Agent use flag                                                        */
+/*----------------------------------------------------------------------------*/
+typedef enum _ua_use_flag_t {
+  UA_USE=0,               /* User-Agent is used.                              */
+  UA_IGN,                 /* User-Agent is disregarded.                       */
+} ua_use_flag_t;
+
+/*----------------------------------------------------------------------------*/
+/* Request parameter maintenance structure                                    */
+/*----------------------------------------------------------------------------*/
 typedef struct _query_string_param_t {
   img_conv_mode_t   mode;
-  char* user_agent;
-  char* name;     /* for EZGET */
-  long offset;    /* for EZGET */
-  long count;     /* for EZGET */
+  char*             user_agent;
+  ua_use_flag_t     ua_flag;
+
+  char*             name;     /* for EZGET */
+  long              offset;    /* for EZGET */
+  long              count;     /* for EZGET */
+  int               width;
+  int               height;
 } query_string_param_t;
 
 /*----------------------------------------------------------------------------*/
-/* for AU CRC table                                                           */
+/* Device_spec when User-Agent is disregarded                                 */
+/*----------------------------------------------------------------------------*/
+static device_table v_ignore_spec = {
+  NULL,
+  "IGN",
+  "IGN",
+  CHXJ_SPEC_HTML,
+  640,
+  480,
+  640,
+  480,
+  1024*1024,
+  1,
+  1,
+  1,
+  0,
+  0,
+  96,
+  96,
+  65536,
+  NULL,
+};
+
+/*----------------------------------------------------------------------------*/
+/* CRC calculation table for AU                                               */
 /*----------------------------------------------------------------------------*/
 static unsigned short  AU_CRC_TBL[256] = {
   0x0000,0x1021,0x2042,0x3063,0x4084,0x50A5,0x60C6,0x70E7,
@@ -85,6 +126,9 @@ static unsigned short  AU_CRC_TBL[256] = {
   0x6E17,0x7E36,0x4E55,0x5E74,0x2E93,0x3EB2,0x0ED1,0x1EF0 
 };
 
+/*----------------------------------------------------------------------------*/
+/* Download page for AU                                                       */
+/*----------------------------------------------------------------------------*/
 static const char* HDML_FIRST_PAGE = 
   "<HDML VERSION=3.0 TTL=0 PUBLIC=TRUE>\r\n"
   "  <NODISPLAY>\r\n"
@@ -108,53 +152,73 @@ static const char* HDML_FAIL_PAGE =
   "  </DISPLAY>\r\n"
   "<HDML>\r\n";
 
-static char* chxj_create_workfile(request_rec* r, 
-                                  mod_chxj_config* conf, 
-                                  const char* user_agent, 
-                                  img_conv_mode_t mode);
-static apr_status_t chxj_create_cache_file(request_rec* r, 
-                                           const char* tmpfile, 
-                                           device_table* spec,
-                                           apr_finfo_t* st,
-                                           img_conv_mode_t mode);
-static apr_status_t chxj_send_cache_file(query_string_param_t* query_string, request_rec* r, const char* tmpfile);
-static query_string_param_t* chxj_get_query_string_param(request_rec *r);
-static unsigned short chxj_add_crc(const char* writedata, apr_size_t witebyte);
-static MagickWand* chxj_fixup_size(MagickWand* magick_wand, 
-                request_rec* r, device_table* spec, img_conv_mode_t mode);
-static MagickWand* chxj_fixup_color(MagickWand* magick_wand, 
-                request_rec* r, device_table* spec, img_conv_mode_t mode);
-static MagickWand* chxj_fixup_depth(MagickWand* magick_wand, 
-                request_rec* r, device_table* spec);
-static MagickWand* chxj_img_down_sizing(MagickWand* magick_wand, 
-                request_rec* r, device_table* spec);
+/*----------------------------------------------------------------------------*/
+/* Prototype declaration                                                      */
+/*----------------------------------------------------------------------------*/
+static char*        s_create_workfile(  request_rec*, mod_chxj_config* , 
+                                        const char*, 
+                                        query_string_param_t*);
+static apr_status_t s_create_cache_file(request_rec* r, 
+                                        const char* tmpfile, 
+                                        device_table* spec,
+                                        apr_finfo_t* st,
+                                        query_string_param_t *qsp);
+static apr_status_t s_send_cache_file(  device_table* spec,
+                                        query_string_param_t* query_string,
+                                        request_rec* r,
+                                        const char* tmpfile);
+static query_string_param_t* s_get_query_string_param(request_rec *r);
+static unsigned short s_add_crc(        const char* writedata, 
+                                        apr_size_t witebyte);
+static MagickWand* s_fixup_size(MagickWand* , 
+                                        request_rec* r, 
+                                        device_table* spec, 
+                                        query_string_param_t *qsp);
+static MagickWand* s_fixup_color(MagickWand* magick_wand, 
+                                 request_rec* r, 
+                                 device_table* spec, 
+                                 img_conv_mode_t mode);
+static MagickWand* s_fixup_depth(MagickWand* magick_wand, 
+                                 request_rec* r, device_table* spec);
+static MagickWand* s_img_down_sizing(MagickWand* magick_wand, 
+                                request_rec* r, device_table* spec);
+static MagickWand* s_add_copyright(
+                          MagickWand* magick_wand,
+                          request_rec* r,
+                          device_table* spec);
+static char* s_create_blob_data(request_rec* r,
+                   device_table* spec,
+                   query_string_param_t *qsp,
+                   char* indata,
+                   apr_size_t* len);
 
-static MagickWand* chxj_add_copyright(MagickWand* magick_wand, request_rec* r, device_table* spec);
+static int s_img_conv_format_from_file(
+                request_rec*          r, 
+                mod_chxj_config*      conf, 
+                const char*           user_agent,
+                query_string_param_t* qsp,
+                device_table*         spec);
 
-
-int
-chxj_img_conv_format(request_rec *r)
+int 
+chxj_img_conv_format_handler(request_rec* r)
 {
-  apr_status_t   rv;
-  apr_finfo_t    st;
-  apr_finfo_t    cache_st;
-  char*          user_agent;
-  device_table*  spec;
-  char*          tmpfile;
-
-  query_string_param_t *qsp = chxj_get_query_string_param(r);
-
-
-  mod_chxj_config* conf;
-
-  ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, 
-                  "chxj_img_conv_format[%s]", r->the_request);
-
-  if (strcasecmp(r->handler, "chxj-picture"))
+  mod_chxj_config*      conf;
+  query_string_param_t* qsp;
+  char*                 user_agent;
+  device_table*         spec;
+  
+  if (strcasecmp(r->handler, "chxj-picture")
+  &&  strcasecmp(r->handler, "chxj-qrcode"))
   {
     return DECLINED;
   }
+  qsp = s_get_query_string_param(r);
+  conf = ap_get_module_config(r->per_dir_config, &chxj_module);
 
+  if (strcasecmp(r->handler, "chxj-qrcode") == 0 &&  conf->image == CHXJ_IMG_OFF)
+  {
+    return DECLINED;
+  }
   if (r->header_only) 
   {
     return DECLINED;
@@ -171,22 +235,108 @@ chxj_img_conv_format(request_rec *r)
   {
     user_agent = (char*)apr_table_get(r->headers_in, HTTP_USER_AGENT);
   }
-  spec = chxj_specified_device(r, user_agent);
+
+  if (qsp->ua_flag == UA_IGN)
+  {
+    spec = &v_ignore_spec;
+  }
+  else
+  {
+    spec = chxj_specified_device(r, user_agent);
+  }
 
   ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "found device_name=[%s]", 
                   spec->device_name);
   ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "User-Agent=[%s]", 
                   user_agent);
+
   if (spec->width == 0 || spec->heigh == 0)
   {
     return HTTP_NOT_FOUND;
   }
 
+  return s_img_conv_format_from_file(r, conf, user_agent, qsp, spec);
+}
+
+
+
+/**
+ * It converts it from ImageData corresponding to each model.
+ *
+ * @param r   [i]
+ * @param src [i]   It is former image binary data.
+ * @param len [i/o] It is length of former image binary data.
+ */
+char*
+chxj_exchange_image(request_rec *r, const char** src, apr_size_t* len)
+{
+  mod_chxj_config*      conf;
+  query_string_param_t* qsp;
+  char*                 user_agent;
+  device_table*         spec;
+  char*                 dst;
+
+
+  qsp = s_get_query_string_param(r);
+  conf = ap_get_module_config(r->per_dir_config, &chxj_module);
+
+  /*--------------------------------------------------------------------------*/
+  /* User-Agent to spec                                                       */
+  /*--------------------------------------------------------------------------*/
+  if (qsp->user_agent != NULL)
+  {
+    user_agent = apr_pstrdup(r->pool, qsp->user_agent);
+  }
+  else
+  {
+    user_agent = (char*)apr_table_get(r->headers_in, HTTP_USER_AGENT);
+  }
+
+  if (qsp->ua_flag == UA_IGN)
+  {
+    spec = &v_ignore_spec;
+  }
+  else
+  {
+    spec = chxj_specified_device(r, user_agent);
+  }
+
+  ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "found device_name=[%s]", 
+                  spec->device_name);
+  ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "User-Agent=[%s]", 
+                  user_agent);
+
+  if (spec->width == 0 || spec->heigh == 0)
+  {
+    *len = 0;
+    return NULL;
+  }
+
+  dst = s_create_blob_data(r, spec, qsp, (char*)*src, len);
+  if (dst == NULL)
+  {
+    *len = 0;
+  }
+  return dst;
+}
+
+static int
+s_img_conv_format_from_file(
+                request_rec*          r, 
+                mod_chxj_config*      conf, 
+                const char*           user_agent,
+                query_string_param_t* qsp,
+                device_table*         spec)
+{
+  apr_status_t   rv;
+  apr_finfo_t    st;
+  apr_finfo_t    cache_st;
+  char*          tmpfile;
+
   /*--------------------------------------------------------------------------*/
   /* Create Workfile Name                                                     */
   /*--------------------------------------------------------------------------*/
-  conf = ap_get_module_config(r->per_dir_config, &chxj_module);
-  tmpfile = chxj_create_workfile(r, conf, user_agent, qsp->mode);
+  tmpfile = s_create_workfile(r, conf, user_agent, qsp);
   ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "workfile=[%s]", tmpfile);
 
   rv = apr_stat(&st, r->filename, APR_FINFO_MIN, r->pool);
@@ -196,20 +346,21 @@ chxj_img_conv_format(request_rec *r)
   }
   ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"found [%s]", r->filename);
   rv = apr_stat(&cache_st, tmpfile, APR_FINFO_MIN, r->pool);
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"found [%s]", r->filename);
   if (rv != APR_SUCCESS || cache_st.ctime < st.mtime)
   {
     /*------------------------------------------------------------------------*/
     /* It tries to make the cash file when it doesn't exist or there is       */
     /* change time later since the making time of the cash file.              */
     /*------------------------------------------------------------------------*/
-    rv = chxj_create_cache_file(r,tmpfile, spec, &st, qsp->mode);
+    rv = s_create_cache_file(r,tmpfile, spec, &st, qsp);
     if (rv != OK)
     {
       return rv;
     }
   }
   ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"color=[%d]", spec->color);
-  rv = chxj_send_cache_file(qsp,r, tmpfile);
+  rv = s_send_cache_file(spec, qsp,r, tmpfile);
   if (rv != OK)
   {
     return rv;
@@ -219,13 +370,19 @@ chxj_img_conv_format(request_rec *r)
   return OK;
 }
 
+
 static apr_status_t
-chxj_create_cache_file(request_rec* r, const char* tmpfile, device_table* spec, apr_finfo_t* st, img_conv_mode_t mode)
+s_create_cache_file(request_rec* r, 
+                       const char* tmpfile, 
+                       device_table* spec, 
+                       apr_finfo_t* st, 
+                       query_string_param_t *qsp)
 {
   apr_status_t       rv;
   apr_size_t         readbyte;
   apr_size_t         writebyte;
   unsigned short     crc;
+  img_conv_mode_t    mode = qsp->mode;
 
   char*              writedata = NULL;
   char*              readdata = NULL;
@@ -236,28 +393,65 @@ chxj_create_cache_file(request_rec* r, const char* tmpfile, device_table* spec,
   MagickBooleanType  status;
   MagickWand*        magick_wand;
 
-  rv = apr_file_open(&fin, 
-                  r->filename, 
-                  APR_READ|APR_BINARY ,
-                  APR_OS_DEFAULT, 
-                  r->pool);
-  if (rv != APR_SUCCESS)
+  if (strcasecmp(r->handler, "chxj-qrcode") == 0)
   {
-    ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
-                    "file open failed.[%s]", r->filename);
-    return HTTP_NOT_FOUND;
-  }
+    /*------------------------------------------------------------------------*/
+    /* QRCODEѤΥեξ                                               */
+    /*------------------------------------------------------------------------*/
+    Doc       doc;
+    Node*     root;
+    qr_code_t qrcode;
+    int       sts;
 
-  readdata = apr_palloc(r->pool, st->size);
-  rv = apr_file_read_full(fin, (void*)readdata, st->size, &readbyte);
-  if (rv != APR_SUCCESS || readbyte != st->size)
+    memset(&doc, 0, sizeof(Doc));
+    memset(&qrcode, 0, sizeof(qr_code_t));
+    doc.r = r;
+    doc.parse_mode = PARSE_MODE_CHTML;
+    qrcode.doc      = &doc;
+    qrcode.r        = r;
+
+    qs_init_malloc(&doc);
+    root = qs_parse_file(&doc, r->filename);
+
+    chxj_qrcode_node_to_qrcode(&qrcode, root);
+    qs_all_free(&doc,QX_LOGMARK);
+    sts = chxj_qrcode_create_image_data(&qrcode, &readdata, &readbyte);
+    if (sts != OK)
+    {
+      ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
+                    "qrcode create failed.");
+      return sts;
+    }
+  }
+  else
+  /*--------------------------------------------------------------------------*/
+  /* ̾Υ᡼եξ                                             */
+  /*--------------------------------------------------------------------------*/
   {
-    ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
-                    "file read failed.[%s]", r->filename);
-    apr_file_close(fin);
-
-    return HTTP_NOT_FOUND;
+    rv = apr_file_open(&fin, 
+                    r->filename, 
+                    APR_READ|APR_BINARY ,
+                    APR_OS_DEFAULT, 
+                    r->pool);
+    if (rv != APR_SUCCESS)
+    {
+      ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
+                      "file open failed.[%s]", r->filename);
+      return HTTP_NOT_FOUND;
+    }
+  
+    readdata = apr_palloc(r->pool, st->size);
+    rv = apr_file_read_full(fin, (void*)readdata, st->size, &readbyte);
+    if (rv != APR_SUCCESS || readbyte != st->size)
+    {
+      ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
+                      "file read failed.[%s]", r->filename);
+      apr_file_close(fin);
+  
+      return HTTP_NOT_FOUND;
+    }
   }
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "start img convert");
 
 
   magick_wand=NewMagickWand();
@@ -272,8 +466,8 @@ chxj_create_cache_file(request_rec* r, const char* tmpfile, device_table* spec,
   /* The size of the image is changed.                                        */
   /*--------------------------------------------------------------------------*/
   ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
-                    "call chxj_fixup_size()");
-  magick_wand = chxj_fixup_size(magick_wand, r, spec, mode);
+                    "call s_fixup_size()");
+  magick_wand = s_fixup_size(magick_wand, r, spec, qsp);
   if (magick_wand == NULL)
   {
     return HTTP_NOT_FOUND;
@@ -283,8 +477,8 @@ chxj_create_cache_file(request_rec* r, const char* tmpfile, device_table* spec,
   /* The colors of the image is changed.                                      */
   /*--------------------------------------------------------------------------*/
   ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
-                    "call chxj_fixup_color()");
-  magick_wand = chxj_fixup_color(magick_wand, r,spec, mode);
+                    "call s_fixup_color()");
+  magick_wand = s_fixup_color(magick_wand, r,spec, mode);
   if (magick_wand == NULL)
   {
     return HTTP_NOT_FOUND;
@@ -294,8 +488,8 @@ chxj_create_cache_file(request_rec* r, const char* tmpfile, device_table* spec,
   /* DEPTH of the image is changed.                                           */
   /*--------------------------------------------------------------------------*/
   ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
-                    "call chxj_fixup_depth()");
-  magick_wand = chxj_fixup_depth(magick_wand, r, spec);
+                    "call s_fixup_depth()");
+  magick_wand = s_fixup_depth(magick_wand, r, spec);
   if (magick_wand == NULL)
   {
     return HTTP_NOT_FOUND;
@@ -326,7 +520,7 @@ chxj_create_cache_file(request_rec* r, const char* tmpfile, device_table* spec,
       return HTTP_NOT_FOUND;
     }
 
-    magick_wand = chxj_img_down_sizing(magick_wand, r, spec);
+    magick_wand = s_img_down_sizing(magick_wand, r, spec);
     if (magick_wand == NULL)
     {
       return HTTP_NOT_FOUND;
@@ -357,7 +551,7 @@ chxj_create_cache_file(request_rec* r, const char* tmpfile, device_table* spec,
       EXIT_MAGICK_ERROR();
       return HTTP_NOT_FOUND;
     }
-    magick_wand = chxj_img_down_sizing(magick_wand, r, spec);
+    magick_wand = s_img_down_sizing(magick_wand, r, spec);
     if (magick_wand == NULL)
     {
       return HTTP_NOT_FOUND;
@@ -389,7 +583,7 @@ chxj_create_cache_file(request_rec* r, const char* tmpfile, device_table* spec,
       return HTTP_NOT_FOUND;
     }
 
-    magick_wand = chxj_img_down_sizing(magick_wand, r, spec);
+    magick_wand = s_img_down_sizing(magick_wand, r, spec);
     if (magick_wand == NULL)
     {
       return HTTP_NOT_FOUND;
@@ -420,7 +614,7 @@ chxj_create_cache_file(request_rec* r, const char* tmpfile, device_table* spec,
       return HTTP_NOT_FOUND;
     }
 
-    magick_wand = chxj_img_down_sizing(magick_wand, r, spec);
+    magick_wand = s_img_down_sizing(magick_wand, r, spec);
     if (magick_wand == NULL)
     {
       return HTTP_NOT_FOUND;
@@ -433,8 +627,8 @@ chxj_create_cache_file(request_rec* r, const char* tmpfile, device_table* spec,
   /* Add Comment (Copyright and so on.)                                       */
   /*--------------------------------------------------------------------------*/
   ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
-                    "call chxj_add_copyright()");
-  magick_wand = chxj_add_copyright(magick_wand, r, spec);
+                    "call s_add_copyright()");
+  magick_wand = s_add_copyright(magick_wand, r, spec);
   if (magick_wand == NULL)
   {
     return HTTP_NOT_FOUND;
@@ -476,7 +670,7 @@ chxj_create_cache_file(request_rec* r, const char* tmpfile, device_table* spec,
   if (spec->html_spec_type == CHXJ_SPEC_XHtml_Mobile_1_0
   ||  spec->html_spec_type == CHXJ_SPEC_Hdml)
   {
-    crc = chxj_add_crc(writedata, writebyte);
+    crc = s_add_crc(writedata, writebyte);
     rv = apr_file_putc((crc >> 8)  & 0xff, fout);
     if (rv != APR_SUCCESS)
     {
@@ -501,10 +695,244 @@ chxj_create_cache_file(request_rec* r, const char* tmpfile, device_table* spec,
   return OK;
 }
 
+
+static char*
+s_create_blob_data(request_rec* r, 
+                   device_table* spec, 
+                   query_string_param_t *qsp,
+                   char* indata,
+                   apr_size_t* len)
+{
+  apr_size_t         writebyte;
+  unsigned short     crc;
+  img_conv_mode_t    mode = qsp->mode;
+
+  char*              writedata = NULL;
+  char*              dst       = NULL;
+
+  MagickBooleanType  status;
+  MagickWand*        magick_wand;
+
+  magick_wand=NewMagickWand();
+  status=MagickReadImageBlob(magick_wand,indata, *len);
+  if (status == MagickFalse)
+  {
+    EXIT_MAGICK_ERROR();
+    return NULL;
+  }
+
+  /*--------------------------------------------------------------------------*/
+  /* The size of the image is changed.                                        */
+  /*--------------------------------------------------------------------------*/
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
+                    "call s_fixup_size()");
+  magick_wand = s_fixup_size(magick_wand, r, spec, qsp);
+  if (magick_wand == NULL)
+  {
+    return NULL;
+  }
+
+  /*--------------------------------------------------------------------------*/
+  /* The colors of the image is changed.                                      */
+  /*--------------------------------------------------------------------------*/
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
+                    "call s_fixup_color()");
+  magick_wand = s_fixup_color(magick_wand, r,spec, mode);
+  if (magick_wand == NULL)
+  {
+    return NULL;
+  }
+
+  /*--------------------------------------------------------------------------*/
+  /* DEPTH of the image is changed.                                           */
+  /*--------------------------------------------------------------------------*/
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
+                    "call s_fixup_depth()");
+  magick_wand = s_fixup_depth(magick_wand, r, spec);
+  if (magick_wand == NULL)
+  {
+    return NULL;
+  }
+
+
+
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
+                    "start convert and compression");
+  if (spec->available_jpeg == 1)
+  {
+    status = MagickSetImageCompression(magick_wand,JPEGCompression);
+    if (status == MagickFalse)
+    {
+      EXIT_MAGICK_ERROR();
+      return NULL;
+    }
+    status = MagickSetImageFormat(magick_wand, "jpg");
+    if (status == MagickFalse)
+    {
+      EXIT_MAGICK_ERROR();
+      return NULL;
+    }
+    status = MagickStripImage(magick_wand);
+    if (status == MagickFalse)
+    {
+      EXIT_MAGICK_ERROR();
+      return NULL;
+    }
+
+    magick_wand = s_img_down_sizing(magick_wand, r, spec);
+    if (magick_wand == NULL)
+    {
+      return NULL;
+    }
+
+    r->content_type = apr_psprintf(r->pool, "image/jpeg");
+    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"convert to jpg");
+  }
+  else
+  if (spec->available_png == 1)
+  {
+    status = MagickSetImageCompression(magick_wand,ZipCompression);
+    if (status == MagickFalse)
+    {
+      EXIT_MAGICK_ERROR();
+      return NULL;
+    }
+    status = MagickSetImageFormat(magick_wand, "png");
+    if (status == MagickFalse)
+    {
+      EXIT_MAGICK_ERROR();
+      return NULL;
+    }
+
+    status = MagickStripImage(magick_wand);
+    if (status == MagickFalse)
+    {
+      EXIT_MAGICK_ERROR();
+      return NULL;
+    }
+    magick_wand = s_img_down_sizing(magick_wand, r, spec);
+    if (magick_wand == NULL)
+    {
+      return NULL;
+    }
+
+    r->content_type = apr_psprintf(r->pool, "image/png");
+    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"convert to png");
+  }
+  else
+  if (spec->available_gif == 1)
+  {
+    status = MagickSetImageCompression(magick_wand,LZWCompression);
+    if (status == MagickFalse)
+    {
+      EXIT_MAGICK_ERROR();
+      return NULL;
+    }
+
+    status = MagickSetImageFormat(magick_wand, "gif");
+    if (status == MagickFalse)
+    {
+      EXIT_MAGICK_ERROR();
+      return NULL;
+    }
+    status = MagickStripImage(magick_wand);
+    if (status == MagickFalse)
+    {
+      EXIT_MAGICK_ERROR();
+      return NULL;
+    }
+
+    magick_wand = s_img_down_sizing(magick_wand, r, spec);
+    if (magick_wand == NULL)
+    {
+      return NULL;
+    }
+
+    r->content_type = apr_psprintf(r->pool, "image/gif");
+    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"convert to gif");
+  }
+  else
+  if (spec->available_bmp2 == 1 || spec->available_bmp4 == 1)
+  {
+    status = MagickSetImageCompression(magick_wand,NoCompression);
+    if (status == MagickFalse)
+    {
+      EXIT_MAGICK_ERROR();
+      return NULL;
+    }
+    status = MagickSetImageFormat(magick_wand, "bmp");
+    if (status == MagickFalse)
+    {
+      EXIT_MAGICK_ERROR();
+      return NULL;
+    }
+    status = MagickStripImage(magick_wand);
+    if (status == MagickFalse)
+    {
+      EXIT_MAGICK_ERROR();
+      return NULL;
+    }
+
+    magick_wand = s_img_down_sizing(magick_wand, r, spec);
+    if (magick_wand == NULL)
+    {
+      return NULL;
+    }
+
+    r->content_type = apr_psprintf(r->pool, "image/bmp");
+    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"convert to bmp(unsupported)");
+  }
+  /*--------------------------------------------------------------------------*/
+  /* Add Comment (Copyright and so on.)                                       */
+  /*--------------------------------------------------------------------------*/
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
+                    "call s_add_copyright()");
+  magick_wand = s_add_copyright(magick_wand, r, spec);
+  if (magick_wand == NULL)
+  {
+    return NULL;
+  }
+
+  writedata=MagickGetImageBlob(magick_wand, &writebyte);
+
+  if (writebyte == 0 || writedata == NULL)
+  {
+    DestroyMagickWand(magick_wand);
+    ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,"convert failure to Jpeg ");
+    return NULL;
+  }
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
+                    "end convert and compression");
+
+
+  
+  dst = apr_palloc(r->pool, writebyte+2);
+  memcpy(dst, writedata, writebyte);
+  /*--------------------------------------------------------------------------*/
+  /* CRC is added for AU for EzGET.                                           */
+  /*--------------------------------------------------------------------------*/
+  if (spec->html_spec_type == CHXJ_SPEC_XHtml_Mobile_1_0
+  ||  spec->html_spec_type == CHXJ_SPEC_Hdml)
+  {
+    crc = s_add_crc(writedata, writebyte);
+    dst[writebyte + 0] = (crc >> 8) & 0xff;
+    dst[writebyte + 1] = (crc     ) & 0xff;
+    writebyte += 2;
+  }
+  DestroyMagickWand(magick_wand);
+
+  *len = writebyte;
+  return dst;
+}
+
 static MagickWand* 
-chxj_fixup_size(MagickWand* magick_wand, request_rec* r, device_table* spec, img_conv_mode_t mode)
+s_fixup_size(MagickWand* magick_wand, 
+                request_rec* r, 
+                device_table* spec, 
+                query_string_param_t *qsp)
 {
   MagickBooleanType  status;
+  img_conv_mode_t mode = qsp->mode;
   int oldw;
   int oldh;
   int neww;
@@ -560,15 +988,18 @@ chxj_fixup_size(MagickWand* magick_wand, request_rec* r, device_table* spec, img
   else
   {
     ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"**** detect normal mode ****");
-    if (neww > c_width)
+    if (qsp->ua_flag != UA_IGN)
     {
-      newh = (int)((double)newh * (double)((double)c_width / (double)neww));
-      neww = (int)((double)neww * (double)((double)c_width / (double)neww));
-    }
-    if (newh > c_heigh)
-    {
-      neww = (int)((double)neww * (double)((double)c_heigh / (double)newh));
-      newh = (int)((double)newh * (double)((double)c_heigh / (double)newh));
+      if (neww > c_width)
+      {
+        newh = (int)((double)newh * (double)((double)c_width / (double)neww));
+        neww = (int)((double)neww * (double)((double)c_width / (double)neww));
+      }
+      if (newh > c_heigh)
+      {
+        neww = (int)((double)neww * (double)((double)c_heigh / (double)newh));
+        newh = (int)((double)newh * (double)((double)c_heigh / (double)newh));
+      }
     }
   }
   ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"convert width=[%d --> %d]", oldw, neww);
@@ -598,6 +1029,19 @@ chxj_fixup_size(MagickWand* magick_wand, request_rec* r, device_table* spec, img
     }
     else
     {
+      if (mode == IMG_CONV_MODE_NORMAL)
+      {
+        if (qsp->width != 0)
+        {
+          ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"convert width=[%d --> %d]", neww, qsp->width);
+          neww = qsp->width;
+        }
+        if (qsp->height != 0)
+        {
+          ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"convert heigh=[%d --> %d]", newh, qsp->height);
+          newh = qsp->height;
+        }
+      }
       status = MagickResizeImage(magick_wand,neww,newh,LanczosFilter,1.0);
       if (status == MagickFalse)
       {
@@ -628,7 +1072,7 @@ chxj_fixup_size(MagickWand* magick_wand, request_rec* r, device_table* spec, img
   return magick_wand;
 }
 static MagickWand*
-chxj_fixup_color(MagickWand* magick_wand, request_rec* r, device_table* spec, img_conv_mode_t mode)
+s_fixup_color(MagickWand* magick_wand, request_rec* r, device_table* spec, img_conv_mode_t mode)
 {
   MagickBooleanType  status;
   ap_log_rerror(APLOG_MARK,APLOG_DEBUG,0,r,"start chxj_fixup_clor()");
@@ -669,7 +1113,7 @@ chxj_fixup_color(MagickWand* magick_wand, request_rec* r, device_table* spec, im
   return magick_wand;
 }
 static MagickWand*
-chxj_fixup_depth(MagickWand* magick_wand, request_rec* r, device_table* spec)
+s_fixup_depth(MagickWand* magick_wand, request_rec* r, device_table* spec)
 {
   MagickBooleanType  status;
 
@@ -746,7 +1190,7 @@ chxj_fixup_depth(MagickWand* magick_wand, request_rec* r, device_table* spec)
 }
 
 static MagickWand*
-chxj_add_copyright(MagickWand* magick_wand, request_rec* r, device_table* spec)
+s_add_copyright(MagickWand* magick_wand, request_rec* r, device_table* spec)
 {
   MagickBooleanType  status;
   mod_chxj_config* conf = ap_get_module_config(r->per_dir_config, &chxj_module);
@@ -797,7 +1241,7 @@ chxj_add_copyright(MagickWand* magick_wand, request_rec* r, device_table* spec)
 }
 
 static MagickWand*
-chxj_img_down_sizing(MagickWand* magick_wand, request_rec* r, device_table* spec)
+s_img_down_sizing(MagickWand* magick_wand, request_rec* r, device_table* spec)
 {
   MagickBooleanType  status;
   unsigned long quality = 70;
@@ -925,7 +1369,7 @@ chxj_img_down_sizing(MagickWand* magick_wand, request_rec* r, device_table* spec
 }
 
 static apr_status_t 
-chxj_send_cache_file(query_string_param_t* query_string, request_rec* r, const char* tmpfile)
+s_send_cache_file(device_table* spec, query_string_param_t* query_string, request_rec* r, const char* tmpfile)
 {
   apr_status_t rv;
   apr_finfo_t  st;
@@ -944,6 +1388,26 @@ chxj_send_cache_file(query_string_param_t* query_string, request_rec* r, const c
   ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"offset:[%ld]", query_string->offset);
   ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"count:[%ld]", query_string->count);
 
+  if (spec->available_jpeg == 1)
+  {
+    r->content_type = apr_psprintf(r->pool, "image/jpeg");
+  }
+  else
+  if (spec->available_png == 1)
+  {
+    r->content_type = apr_psprintf(r->pool, "image/png");
+  }
+  else
+  if (spec->available_gif == 1)
+  {
+    r->content_type = apr_psprintf(r->pool, "image/gif");
+  }
+  else
+  if (spec->available_bmp2 == 1 || spec->available_bmp4 == 1)
+  {
+    r->content_type = apr_psprintf(r->pool, "image/bmp");
+  }
+
   if (query_string->mode != IMG_CONV_MODE_EZGET && query_string->name == NULL)
   {
     contentLength = apr_psprintf(r->pool, "%d", (int)st.size);
@@ -1039,7 +1503,11 @@ chxj_send_cache_file(query_string_param_t* query_string, request_rec* r, const c
 
 
 static char*
-chxj_create_workfile(request_rec* r, mod_chxj_config* conf, const char* user_agent, img_conv_mode_t mode)
+s_create_workfile(
+                request_rec*          r, 
+                mod_chxj_config*      conf, 
+                const char*           user_agent, 
+                query_string_param_t *qsp)
 {
   int ii;
   int jj;
@@ -1048,7 +1516,7 @@ chxj_create_workfile(request_rec* r, mod_chxj_config* conf, const char* user_age
   char* fname;
 
   memset(w, 0, 256);
-  switch (mode) {
+  switch (qsp->mode) {
   case IMG_CONV_MODE_THUMBNAIL:
     fname = apr_psprintf(r->pool, "%s.%s.thumbnail", r->filename, user_agent);
     ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mode=thumbnail [%s]", fname);
@@ -1061,9 +1529,21 @@ chxj_create_workfile(request_rec* r, mod_chxj_config* conf, const char* user_age
   case IMG_CONV_MODE_NORMAL:
   default:
     fname = apr_psprintf(r->pool, "%s.%s", r->filename, user_agent);
+    if (qsp->width != 0)
+    {
+      fname = apr_psprintf(r->pool, "%s.w%d", fname, qsp->width);
+    }
+    if (qsp->height != 0)
+    {
+      fname = apr_psprintf(r->pool, "%s.h%d", fname, qsp->height);
+    }
     ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "mode=normal [%s]", fname);
     break;
   }
+  if (qsp->ua_flag == UA_IGN)
+  {
+    fname = apr_psprintf(r->pool, "%s.IGN", fname);
+  }
 
   len = strlen(fname);
   jj=0;
@@ -1087,7 +1567,7 @@ chxj_create_workfile(request_rec* r, mod_chxj_config* conf, const char* user_age
 }
 
 static unsigned short
-chxj_add_crc(const char* writedata, apr_size_t writebyte)
+s_add_crc(const char* writedata, apr_size_t writebyte)
 {
   unsigned short crc = 0xffff;
   apr_size_t     ii;
@@ -1107,39 +1587,24 @@ chxj_trans_name(request_rec *r)
   const char* ccp;
   char* docroot;
   int len;
-  ap_regex_t *regexp;
-  ap_regmatch_t match[10];
   apr_finfo_t st;
   apr_status_t rv;
   mod_chxj_config* conf;
-  int rtn;
   int ii;
-  char*      ext[4] = {
+  char*      ext[] = {
           "jpg",
           "png",
           "bmp",
           "gif",
+          "qrc",    /* QRCodeѥեγĥ */
   };
   char*    fname;
 
   conf = ap_get_module_config(r->per_dir_config, &chxj_module);
-  if (conf->image_uri == NULL)
+  if (conf->image != CHXJ_IMG_ON)
   {
     return DECLINED;
   }
-
-  regexp = ap_pregcomp(r->pool, (const char*)conf->image_uri, AP_REG_EXTENDED|AP_REG_ICASE);
-  if (regexp == NULL)
-  {
-    return DECLINED;
-  }
-
-  rtn = ap_regexec(regexp, r->uri, regexp->re_nsub + 1, match, 0);
-  if (rtn != 0)
-  {
-    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "Not match URI[%s]", r->uri);
-    return DECLINED;
-  }
   ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "Match URI[%s]", r->uri);
 
   if (r->filename == NULL) 
@@ -1170,7 +1635,8 @@ chxj_trans_name(request_rec *r)
   }
   ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "URI[%s]", r->filename);
 
-  for (ii=0; ii<4; ii++) 
+
+  for (ii=0; ii<5; ii++) 
   {
     fname = apr_psprintf(r->pool, "%s.%s", r->filename, ext[ii]);
     ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "search [%s]", fname);
@@ -1182,14 +1648,26 @@ chxj_trans_name(request_rec *r)
     }
     fname = NULL;
   }
-  if (fname == NULL)
+  if (r->handler == NULL || strcasecmp(r->handler, "chxj-qrcode") != 0)
   {
-    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "NotFound [%s]", r->filename);
-    return HTTP_NOT_FOUND;
+    if (fname == NULL)
+    {
+      ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "NotFound [%s]", r->filename);
+      return DECLINED;
+    }
+  
+    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "Found [%s]", fname);
+    r->filename = apr_psprintf(r->pool, "%s", fname);
+  
+    if (strcasecmp("qrc", ext[ii]) == 0)
+    {
+      r->handler = apr_psprintf(r->pool, "chxj-qrcode");
+    }
+    else
+    {
+      r->handler = apr_psprintf(r->pool, "chxj-picture");
+    }
   }
-  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "Found [%s]", fname);
-  r->filename = apr_psprintf(r->pool, "%s", fname);
-  r->handler = apr_psprintf(r->pool, "chxj-picture");
   return OK;
 }
 
@@ -1201,7 +1679,7 @@ chxj_trans_name(request_rec *r)
  * @param r   [i]
  */
 static query_string_param_t*
-chxj_get_query_string_param(request_rec *r)
+s_get_query_string_param(request_rec *r)
 {
   char* pair;
   char* name;
@@ -1214,15 +1692,13 @@ chxj_get_query_string_param(request_rec *r)
   param = apr_palloc(r->pool, sizeof(query_string_param_t));
   param->mode       = IMG_CONV_MODE_NORMAL;
   param->user_agent = NULL;
+  param->ua_flag    = UA_USE;
   param->name       = NULL;
   param->offset     = 0;
   param->count      = 0;
+  param->width      = 0;
+  param->height     = 0;
 
-  ap_log_rerror(APLOG_MARK,
-                APLOG_DEBUG,
-                0, 
-                r, 
-                "query_string[%s]", s);
   if (s == NULL)
   {
     return param;
@@ -1239,14 +1715,14 @@ chxj_get_query_string_param(request_rec *r)
 
     name  = apr_strtok(pair, "=", &vstate);
     value = apr_strtok(NULL, "=", &vstate);
-    if (strcasecmp(name, "mode") == 0 && value != NULL) 
+    if ((strcasecmp(name, "mode") == 0 || strcasecmp(name, "m") == 0) && value != NULL) 
     {
-      if (strcasecmp(value, "thumbnail") == 0)
+      if (strcasecmp(value, "thumbnail") == 0 || strcasecmp(value, "tb") == 0)
       {
         param->mode = IMG_CONV_MODE_THUMBNAIL;
       }
       else
-      if (strcasecmp(value, "WP") == 0)
+      if (strcasecmp(value, "WP") == 0 || strcasecmp(value, "WallPaper") == 0)
       {
         param->mode = IMG_CONV_MODE_WALLPAPER;
       }
@@ -1257,9 +1733,13 @@ chxj_get_query_string_param(request_rec *r)
       }
     }
     else
-    if (strcasecmp(name, "user-agent") == 0 && value != NULL)
+    if ((strcasecmp(name, "ua") == 0 || strcasecmp(name, "user-agent") == 0) && value != NULL)
     {
       ap_unescape_url(value);
+      if (strcasecmp(value, "IGN") == 0)
+      {
+        param->ua_flag = UA_IGN;
+      }
       param->user_agent = apr_pstrdup(r->pool, value);
     }
     else
@@ -1283,6 +1763,22 @@ chxj_get_query_string_param(request_rec *r)
         param->count = chxj_atoi(value);
       }
     }
+    else
+    if (strcasecmp(name, "w") == 0 && value != NULL)
+    {
+      if (chxj_chk_numeric(value) == 0)
+      {
+        param->width = chxj_atoi(value);
+      }
+    }
+    else
+    if (strcasecmp(name, "h") == 0 && value != NULL)
+    {
+      if (chxj_chk_numeric(value) == 0)
+      {
+        param->height = chxj_atoi(value);
+      }
+    }
   }
 
   if (param->mode == IMG_CONV_MODE_NORMAL && param->name != NULL)
