@@ -38,16 +38,21 @@ chxj_specified_device(request_rec* r, const char* user_agent)
   int rtn;
   char* device_id;
 
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "start chxj_specified_device()");
   conf = ap_get_module_config(r->per_dir_config, &chxj_module);
   for (dtl = conf->devices; dtl; dtl = dtl->next) 
   {
     if (dtl->pattern == NULL)
     {
+      ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "pattern is null");
       continue;
     }
+
+    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "pattern is [%s]", dtl->pattern);
     regexp = ap_pregcomp(r->pool, (const char*)dtl->pattern, AP_REG_EXTENDED|AP_REG_ICASE);
     if (regexp == NULL) 
     {
+      ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "compile failed.");
       return returnType;
     }
 
@@ -82,9 +87,11 @@ chxj_specified_device(request_rec* r, const char* user_agent)
     ap_pregfree(r->pool, regexp);
     if (returnType != &UNKNOWN_DEVICE) 
     {
+      ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "end chxj_specified_device()");
       return returnType;
     }
   }
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "end chxj_specified_device()");
 
   return returnType;
 }
