@@ -19,6 +19,7 @@
 #include "chxj_hdml.h"
 #include "chxj_dump.h"
 #include "chxj_img_conv.h"
+#include "chxj_qr_code.h"
 
 
 static char* xhtml_1_0_node_exchange    (Xhtml* xhtml, Node* node, int indent);
@@ -82,10 +83,23 @@ chxj_exchange_xhtml_mobile_1_0(
   Xhtml     xhtml;
   Doc       doc;
 
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"start chxj_exchange_xhtml_mobile_1_0()");
+
+  /*--------------------------------------------------------------------------*/
+  /* If qrcode xml                                                            */
+  /*--------------------------------------------------------------------------*/
+  *dstlen = srclen;
+  dst = chxj_qr_code_blob_handler(r, src, (size_t*)dstlen);
+  if (dst != NULL)
+  {
+    ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"i found qrcode xml");
+    return dst;
+  }
+  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"not found qrcode xml");
+
   /*--------------------------------------------------------------------------*/
   /* The XHTML structure is initialized.                                      */
   /*--------------------------------------------------------------------------*/
-  ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,"start chxj_exchange_xhtml_mobile_1_0()");
   chxj_init_xhtml(&xhtml, &doc, r, spec);
   ap_set_content_type(r, "text/html; charset=Windows-31J");
 
@@ -358,7 +372,17 @@ xhtml_1_0_node_exchange(Xhtml* xhtml, Node* node, int indent)
           ap_set_content_type(r, "text/x-hdml; charset=Shift_JIS");
         }
         ap_log_rerror(APLOG_MARK, APLOG_DEBUG,0,r, "chxj:if tag is mine");
-        xhtml_1_0_chxjif_tag(xhtml, child);
+
+        char* parse_attr = NULL;
+        parse_attr = qs_get_parse_attr(doc, child, r);
+        if (parse_attr != NULL && strcasecmp(parse_attr, "true") == 0)
+        {
+          xhtml_1_0_node_exchange (xhtml, child, indent+1);
+        }
+        else
+        {
+          xhtml_1_0_chxjif_tag(xhtml, child);
+        }
       }
     }
     /*------------------------------------------------------------------------*/
