@@ -53,6 +53,7 @@
 #include "chxj_jhtml.h"
 
 #include "chxj_img_conv_format.h"
+#include "chxj_qr_code.h"
 
 
 #ifdef PACKAGE_NAME
@@ -116,8 +117,6 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len)
       /*----------------------------------------------------------------------*/
       ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "select DoCoMo i-Mode 1.0 ");
       dst = chxj_exchange_chtml10(r, spec, *src, *len, len);
-      *len = strlen(dst);
-      ap_set_content_type(r, "text/html; charset=Windows-31J");
 
     }
     else
@@ -128,8 +127,6 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len)
       /*----------------------------------------------------------------------*/
       ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "select DoCoMo i-Mode 2.0 ");
       dst = chxj_exchange_chtml20(r, spec, *src, *len, len);
-      *len = strlen(dst);
-      ap_set_content_type(r, "text/html; charset=Windows-31J");
     }
     else 
     if (spec->html_spec_type == CHXJ_SPEC_Chtml_3_0) 
@@ -139,8 +136,6 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len)
       /*----------------------------------------------------------------------*/
       ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "select DoCoMo i-Mode 3.0 ");
       dst = chxj_exchange_chtml30(r, spec, *src, *len, len);
-      *len = strlen(dst);
-      ap_set_content_type(r, "text/html; charset=Windows-31J");
     }
     else
     if (spec->html_spec_type == CHXJ_SPEC_Chtml_4_0) 
@@ -150,8 +145,6 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len)
       /*----------------------------------------------------------------------*/
       ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "select DoCoMo i-Mode 4.0 ");
       dst = chxj_exchange_chtml30(r, spec, *src, *len, len);
-      *len = strlen(dst);
-      ap_set_content_type(r, "text/html; charset=Windows-31J");
     }
     else 
     if (spec->html_spec_type == CHXJ_SPEC_Chtml_5_0) 
@@ -161,8 +154,6 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len)
       /*----------------------------------------------------------------------*/
       ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "select DoCoMo i-Mode 5.0 ");
       dst = chxj_exchange_chtml30(r, spec, *src, *len, len);
-      *len = strlen(dst);
-      ap_set_content_type(r, "text/html; charset=Windows-31J");
     }
     else
     if (spec->html_spec_type == CHXJ_SPEC_XHtml_Mobile_1_0) 
@@ -172,7 +163,6 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len)
       /*----------------------------------------------------------------------*/
       ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "select XHTML Mobile 1.0");
       dst = chxj_exchange_xhtml_mobile_1_0(r, spec, *src, *len, len);
-      *len = strlen(dst);
     }
     else
     if (spec->html_spec_type == CHXJ_SPEC_Hdml) 
@@ -182,7 +172,6 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len)
       /*----------------------------------------------------------------------*/
       ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "select HDML");
       dst = chxj_exchange_hdml(r, spec, *src, *len, len);
-      ap_set_content_type(r, "text/x-hdml; charset=Shift_JIS");
     }
     else
     if (spec->html_spec_type == CHXJ_SPEC_Jhtml) 
@@ -192,8 +181,6 @@ chxj_exchange(request_rec *r, const char** src, apr_size_t* len)
       /*----------------------------------------------------------------------*/
       ap_log_rerror(APLOG_MARK,APLOG_DEBUG, 0, r, "select JHTML");
       dst = chxj_exchange_jhtml(r, spec, *src, *len, len);
-      *len = strlen(dst);
-      ap_set_content_type(r, "text/html; charset=Windows-31J");
     }
     else 
     {
@@ -284,6 +271,7 @@ chxj_input_exchange(request_rec *r, const char** src, apr_size_t* len)
   return result;
 }
 
+
 /**
  * The received data is returned to the filter.
  *
@@ -382,6 +370,18 @@ chxj_output_filter(ap_filter_t *f, apr_bucket_brigade *bb)
             ctx->len += 1;
           }
         }
+        if (strncmp(r->content_type, "image/", 6) == 0)
+        {
+          if (ctx->len)
+          {
+            char* tmp = apr_palloc(r->pool, ctx->len + 1);
+            memset(tmp, 0, ctx->len + 1);
+            memcpy(tmp, ctx->buffer, ctx->len);
+            ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "input data=[%s]", tmp);
+            ctx->buffer = chxj_exchange_image(r, (const char**)&tmp, (apr_size_t*)&ctx->len);
+            ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, "output data=[%.*s]", ctx->len,ctx->buffer);
+          }
+        }
         contentLength = apr_psprintf(r->pool, "%d", ctx->len);
         apr_table_setn(r->headers_out, "Content-Length", contentLength);
         
@@ -657,11 +657,6 @@ chxj_config_server_create(apr_pool_t *p, server_rec *s)
   return gc;
 }
 
-static int chxj_convert_images(request_rec *r)
-{
-  return chxj_img_conv_format(r);
-}
-
 
 static int
 chxj_translate_name(request_rec *r)
@@ -691,7 +686,8 @@ chxj_register_hooks(apr_pool_t *p)
                       chxj_input_filter, 
                       NULL, 
                       AP_FTYPE_RESOURCE);
-  ap_hook_handler(chxj_convert_images, NULL, NULL, APR_HOOK_MIDDLE);
+  ap_hook_handler(chxj_img_conv_format_handler, NULL, NULL, APR_HOOK_MIDDLE);
+  ap_hook_handler(chxj_qr_code_handler, NULL, NULL, APR_HOOK_MIDDLE);
   ap_hook_translate_name(chxj_translate_name, NULL, NULL, APR_HOOK_MIDDLE);
 }
 
@@ -708,15 +704,88 @@ chxj_create_per_dir_config(apr_pool_t *p, char *arg)
 
   conf = apr_pcalloc(p, sizeof(mod_chxj_config));
   conf->device_data_file = NULL;
+  conf->devices          = NULL;
   conf->emoji_data_file  = NULL;
-  conf->image_uri        = NULL;
+  conf->emoji            = NULL;
+  conf->emoji_tail       = NULL;
+  conf->image            = CHXJ_IMG_OFF;
   conf->image_cache_dir  = apr_psprintf(p, "%s",DEFAULT_IMAGE_CACHE_DIR);
 
   /* Default is copyleft */
   conf->image_copyright = NULL; 
   return conf;
 }
+/*
+ *  Merge per-directory CHXJ configurations
+ */
+static void*
+chxj_merge_per_dir_config(apr_pool_t *p, void *basev, void *addv)
+{
+  mod_chxj_config *base = (mod_chxj_config*)basev;
+  mod_chxj_config *add  = (mod_chxj_config*)addv;
+  mod_chxj_config *mrg  = (mod_chxj_config*)apr_palloc(p, sizeof(mod_chxj_config));
 
+  mrg->device_data_file = NULL;
+  mrg->devices          = NULL;
+  mrg->emoji_data_file  = NULL;
+  mrg->image            = CHXJ_IMG_OFF;
+  mrg->image_cache_dir  = NULL;
+  mrg->image_copyright  = NULL;
+  mrg->emoji            = NULL;
+  mrg->emoji_tail       = NULL;
+  if (add->device_data_file == NULL)
+  {
+    mrg->devices = base->devices;
+    mrg->device_data_file = apr_pstrdup(p, base->device_data_file);
+  }
+  else
+  {
+    mrg->devices = add->devices;
+    mrg->device_data_file = apr_pstrdup(p, add->device_data_file);
+  }
+
+  if (add->emoji_data_file == NULL)
+  {
+    mrg->emoji = base->emoji;
+    mrg->emoji_tail = base->emoji_tail;
+    mrg->emoji_data_file = apr_pstrdup(p, base->emoji_data_file);
+  }
+  else
+  {
+    mrg->emoji = add->emoji;
+    mrg->emoji_tail = add->emoji_tail;
+    mrg->emoji_data_file = apr_pstrdup(p, add->emoji_data_file);
+  }
+
+  if (add->image == CHXJ_IMG_OFF)
+  {
+    mrg->image = base->image;
+  }
+  else
+  {
+    mrg->image = add->image;
+  }
+
+  if (strcasecmp(add->image_cache_dir ,DEFAULT_IMAGE_CACHE_DIR)==0)
+  {
+    mrg->image_cache_dir = apr_pstrdup(p, base->image_cache_dir);
+  }
+  else
+  {
+    mrg->image_cache_dir = apr_pstrdup(p, add->image_cache_dir);
+  }
+
+  if (add->image_copyright == NULL && base->image_copyright != NULL)
+  {
+    mrg->image_copyright = apr_pstrdup(p, base->image_copyright);
+  }
+  else
+  if (add->image_copyright != NULL)
+  {
+    mrg->image_copyright = apr_pstrdup(p, add->image_copyright);
+  }
+  return mrg;
+}
 /**
  * The device definition file is loaded. 
  *
@@ -781,7 +850,7 @@ cmd_load_emoji_data(cmd_parms *parms, void *mconfig, const char* arg)
 }
 
 static const char* 
-cmd_set_image_uri(cmd_parms *parms, void *mconfig, const char* arg) 
+cmd_set_image_engine(cmd_parms *parms, void *mconfig, const char* arg) 
 {
   mod_chxj_config* conf;
   Doc doc;
@@ -792,7 +861,14 @@ cmd_set_image_uri(cmd_parms *parms, void *mconfig, const char* arg)
   }
 
   conf = (mod_chxj_config*)mconfig;
-  conf->image_uri = apr_pstrdup(parms->pool, arg);
+  if (strcasecmp("ON", arg) == 0)
+  {
+    conf->image = CHXJ_IMG_ON;
+  }
+  else
+  {
+    conf->image = CHXJ_IMG_OFF;
+  }
   return NULL;
 }
 
@@ -829,6 +905,7 @@ cmd_set_image_copyright(cmd_parms *parms, void *mconfig, const char* arg)
   return NULL;
 }
 
+
 static const command_rec cmds[] = {
   AP_INIT_TAKE1(
     "ChxjLoadDeviceData",
@@ -843,8 +920,8 @@ static const command_rec cmds[] = {
     OR_ALL,
     "Load Emoji Data"),
   AP_INIT_TAKE1(
-    "ChxjImageUri",
-    cmd_set_image_uri,
+    "ChxjImageEngine",
+    cmd_set_image_engine,
     NULL,
     OR_ALL,
     "Convert Target URI"),
@@ -871,7 +948,7 @@ module AP_MODULE_DECLARE_DATA chxj_module =
 {
   STANDARD20_MODULE_STUFF, 
   chxj_create_per_dir_config,          /* create per-dir    config structures */
-  NULL,                                /* merge  per-dir    config structures */
+  chxj_merge_per_dir_config,           /* merge  per-dir    config structures */
   chxj_config_server_create,           /* create per-server config structures */
   NULL,                                /* merge  per-server config structures */
   cmds,                                /* table of config file commands       */
