@@ -18,7 +18,7 @@
 #define __QS_IGNORE_SP_H__
 #include "qs_parse_string.h"
 
-QS_EXPORT int qs_ignore_sp(Doc* doc, const char*, int);
-QS_EXPORT int qs_ignore_sp_and_quote(Doc* doc, const char* s, int len) ;
+int qs_ignore_sp(Doc* doc, const char*, int);
+int qs_ignore_sp_and_quote(Doc* doc, const char* s, int len) ;
 
 #endif
